# [line-too-long, missing-module-docstring, undefined-all-variable, undefined-all-variable, undefined-all-variable, undefined-all-variable, undefined-all-variable]
# ↓ equivalent to __all__ = ("C", "O", "N", "S", "T")
__all__ = tuple("CONST")

CONST = 42
