[EXCEPTIONS]
overgeneral-exceptions=builtins.BaseException,
                       builtins.Exception,
                       functional.b.broad_exception.broad_exception_caught.CustomBroadException,
                       broad_exception_caught.CustomBroadException
