# pylint: disable=too-few-public-methods,missing-docstring,used-before-assignment,import-error,unused-import
# pylint: disable=wrong-import-position


@decorator
class DecoratedClass:
    decorator: int

import decorator
